@@ -27,7 +27,7 @@ from xonsh.inspectors import Inspector
 from xonsh.aliases import Aliases, make_default_aliases
 from xonsh.environ import Env, default_env, locate_binary
 from xonsh.jobs import add_job
-from xonsh.platform import ON_POSIX, ON_WINDOWS
+from xonsh.platform import ON_POSIX, ON_WINDOWS, ON_WSL
 from xonsh.proc import (
     PopenThread,
     ProcProxyThread,
@@ -377,18 +377,18 @@ class SubprocSpec:
     executed.
     """
 
-    kwnames = ("stdin", "stdout", "stderr", "universal_newlines")
+    kwnames = ("stdin", "stdout", "stderr", "universal_newlines", "close_fds")
 
     def __init__(
         self,
         cmd,
-        *,
         cls=subprocess.Popen,
         stdin=None,
         stdout=None,
         stderr=None,
         universal_newlines=False,
-        captured=False
+        close_fds=False,
+        captured=False,
     ):
         """
         Parameters
@@ -405,6 +405,9 @@ class SubprocSpec:
             Popen file descriptor or flag for stderr.
         universal_newlines : bool
             Whether or not to use universal newlines.
+        close_fds : bool
+            Whether or not to close the file descriptiors when the
+            process exits.
         captured : bool or str, optional
             The flag for if the subprocess is captured, may be one of:
             False for $[], 'stdout' for $(), 'hiddenobject' for ![], or
@@ -445,6 +448,7 @@ class SubprocSpec:
         self.stdout = stdout
         self.stderr = stderr
         self.universal_newlines = universal_newlines
+        self.close_fds = close_fds
         self.captured = captured
         # pure attrs
         self.args = list(cmd)
@@ -531,6 +535,8 @@ class SubprocSpec:
 
     def run(self, *, pipeline_group=None):
         """Launches the subprocess and returns the object."""
+        event_name = self._cmd_event_name()
+        self._pre_run_event_fire(event_name)
         kwargs = {n: getattr(self, n) for n in self.kwnames}
         self.prep_env(kwargs)
         self.prep_preexec_fn(kwargs, pipeline_group=pipeline_group)
@@ -545,6 +551,7 @@ class SubprocSpec:
         p.last_in_pipeline = self.last_in_pipeline
         p.captured_stdout = self.captured_stdout
         p.captured_stderr = self.captured_stderr
+        self._post_run_event_fire(event_name, p)
         return p
 
     def _run_binary(self, kwargs):
@@ -579,7 +586,9 @@ class SubprocSpec:
             return
         if not builtins.__xonsh__.env.get("XONSH_INTERACTIVE"):
             return
-        if pipeline_group is None:
+        if pipeline_group is None or ON_WSL:
+            # If there is no pipeline group
+            # or the platform is windows subsystem for linux (WSL)
             xonsh_preexec_fn = no_pg_xonsh_preexec_fn
         else:
 
@@ -598,6 +607,26 @@ class SubprocSpec:
         for i in range(len(cmd)):
             cmd[i] = cmd[i].replace("\0", "\\0")
 
+    def _cmd_event_name(self):
+        if callable(self.alias):
+            return self.alias.__name__
+        elif self.binary_loc is None:
+            return "<not-found>"
+        else:
+            return os.path.basename(self.binary_loc)
+
+    def _pre_run_event_fire(self, name):
+        event_name = "on_pre_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self)
+
+    def _post_run_event_fire(self, name, proc):
+        event_name = "on_post_spec_run_" + name
+        if events.exists(event_name):
+            event = getattr(events, event_name)
+            event.fire(spec=self, proc=proc)
+
     #
     # Building methods
     #
@@ -902,6 +931,12 @@ def run_subproc(cmds, captured=False):
     if _should_set_title(captured=captured):
         # set title here to get currently executing command
         pause_call_resume(proc, builtins.__xonsh__.shell.settitle)
+    else:
+        # for some reason, some programs are in a stopped state when the flow
+        # reaches this point, hence a SIGCONT should be sent to `proc` to make
+        # sure that the shell doesn't hang. This `pause_call_resume` invocation
+        # does this
+        pause_call_resume(proc, int)
     # create command or return if backgrounding.
     if background:
         return
@@ -1062,6 +1097,8 @@ def convert_macro_arg(raw_arg, kind, glbs, locs, *, name="<arg>", macroname="<ma
         if locs is not None:
             ctx |= set(locs.keys())
         mode = mode or "eval"
+        if mode != "eval" and not raw_arg.endswith("\n"):
+            raw_arg += "\n"
         arg = execer.parse(raw_arg, ctx, mode=mode, filename=filename)
     elif kind is types.CodeType or kind is compile:  # NOQA
         mode = mode or "eval"
