@@ -91,9 +91,11 @@ def _collapsed_pwd():
     pwd = _replace_home_cwd().split(sep)
     l = len(pwd)
     leader = sep if l > 0 and len(pwd[0]) == 0 else ""
-    base = [i[0]
-            if ix != l - 1 and i[0] != '.' else i[0:2]
-            if ix != l - 1 else i for ix, i in enumerate(pwd) if len(i) > 0]
+    base = [
+        i[0] if ix != l - 1 and i[0] != "." else i[0:2] if ix != l - 1 else i
+        for ix, i in enumerate(pwd)
+        if len(i) > 0
+    ]
     return leader + sep.join(base)
 
 
@@ -151,16 +153,37 @@ def _dynamically_collapsed_pwd():
 # amalgamated os
 # amalgamated builtins
 # amalgamated xonsh.platform
-def env_name(pre_chars="(", post_chars=")"):
-    """Extract the current environment name from $VIRTUAL_ENV or
-    $CONDA_DEFAULT_ENV if that is set
+def find_env_name():
+    """Finds the current environment name from $VIRTUAL_ENV or
+    $CONDA_DEFAULT_ENV if that is set.
     """
     env_path = builtins.__xonsh__.env.get("VIRTUAL_ENV", "")
     if len(env_path) == 0 and xp.ON_ANACONDA:
         env_path = builtins.__xonsh__.env.get("CONDA_DEFAULT_ENV", "")
     env_name = os.path.basename(env_path)
-    if env_name:
-        return pre_chars + env_name + post_chars
+    return env_name
+
+
+def env_name():
+    """Returns the current env_name if it non-empty, surrounded by the
+    ``{env_prefix}`` and ``{env_postfix}`` fields.
+    """
+    env_name = find_env_name()
+    if (
+        builtins.__xonsh__.env.get("VIRTUAL_ENV_DISABLE_PROMPT")
+        or not env_name
+    ):
+        # env name prompt printing disabled, or no environment; just return
+        return
+
+    venv_prompt = builtins.__xonsh__.env.get("VIRTUAL_ENV_PROMPT")
+    if venv_prompt is not None:
+        return venv_prompt
+    else:
+        pf = builtins.__xonsh__.shell.prompt_formatter
+        pre = pf._get_field_value("env_prefix")
+        post = pf._get_field_value("env_postfix")
+        return pre + env_name + post
 
 
 def vte_new_tab_cwd():
@@ -752,6 +775,8 @@ def PROMPT_FIELDS():
         branch_bg_color=branch_bg_color,
         current_job=_current_job,
         env_name=env_name,
+        env_prefix="(",
+        env_postfix=") ",
         vte_new_tab_cwd=vte_new_tab_cwd,
         gitstatus=gitstatus_prompt,
     )
@@ -761,19 +786,20 @@ def default_prompt():
     """Creates a new instance of the default prompt."""
     if xp.ON_CYGWIN or xp.ON_MSYS:
         dp = (
-            "{env_name:{} }{BOLD_GREEN}{user}@{hostname}"
+            "{env_name}"
+            "{BOLD_GREEN}{user}@{hostname}"
             "{BOLD_BLUE} {cwd} {prompt_end}{NO_COLOR} "
         )
     elif xp.ON_WINDOWS and not xp.win_ansi_support():
         dp = (
-            "{env_name:{} }"
+            "{env_name}"
             "{BOLD_INTENSE_GREEN}{user}@{hostname}{BOLD_INTENSE_CYAN} "
             "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
             "{BOLD_INTENSE_CYAN}{prompt_end}{NO_COLOR} "
         )
     else:
         dp = (
-            "{env_name:{} }"
+            "{env_name}"
             "{BOLD_GREEN}{user}@{hostname}{BOLD_BLUE} "
             "{cwd}{branch_color}{curr_branch: {}}{NO_COLOR} "
             "{BOLD_BLUE}{prompt_end}{NO_COLOR} "
