@@ -616,6 +616,16 @@ def subexpr_before_unbalanced(expr, ltok, rtok):
     return subexpr
 
 
+@lazyobject
+def STARTING_WHITESPACE_RE():
+    return re.compile(r"^(\s*)")
+
+
+def starting_whitespace(s):
+    """Returns the whitespace at the start of a string"""
+    return STARTING_WHITESPACE_RE.match(s).group(1)
+
+
 def decode(s, encoding=None):
     encoding = encoding or DEFAULT_ENCODING
     return s.decode(encoding, "replace")
@@ -756,7 +766,7 @@ def _yield_accessible_unix_file_names(path):
         try:
             if file_.is_file() and os.access(file_.path, os.X_OK):
                 yield file_.name
-        except (FileNotFoundError, NotADirectoryError):
+        except OSError:
             # broken Symlink are neither dir not files
             pass
 
@@ -790,7 +800,11 @@ def _executables_in_windows(path):
                     yield fname
     else:
         for x in scandir(path):
-            if x.is_file():
+            try:
+                is_file = x.is_file()
+            except OSError:
+                continue
+            if is_file:
                 fname = x.name
             else:
                 continue
@@ -1075,6 +1089,11 @@ def swap_values(d, updates, default=_DEFAULT_SENTINEL):
 #
 
 
+def detype(x):
+    """This assumes that the object has a detype method, and calls that."""
+    return x.detype()
+
+
 def is_int(x):
     """Tests if something is an integer"""
     return isinstance(x, int)
@@ -1120,6 +1139,11 @@ def always_false(x):
     return False
 
 
+def always_none(x):
+    """Returns None"""
+    return None
+
+
 def ensure_string(x):
     """Returns a string if x is not a string, and x if it already is."""
     return str(x)
@@ -1689,6 +1713,12 @@ def history_tuple_to_str(x):
     return "{0} {1}".format(*x)
 
 
+def all_permutations(iterable):
+    """Yeilds all permutations, not just those of a specified length"""
+    for r in range(1, len(iterable) + 1):
+        yield from itertools.permutations(iterable, r=r)
+
+
 def format_color(string, **kwargs):
     """Formats strings that may contain colors. This simply dispatches to the
     shell instances method of the same name. The results of this function should
@@ -1979,6 +2009,38 @@ def RE_COMPLETE_STRING():
     return re.compile(ptrn, re.DOTALL)
 
 
+def strip_simple_quotes(s):
+    """Gets rid of single quotes, double quotes, single triple quotes, and
+    single double quotes from a string, if present front and back of a string.
+    Otherwiswe, does nothing.
+    """
+    starts_single = s.startswith("'")
+    starts_double = s.startswith('"')
+    if not starts_single and not starts_double:
+        return s
+    elif starts_single:
+        ends_single = s.endswith("'")
+        if not ends_single:
+            return s
+        elif s.startswith("'''") and s.endswith("'''") and len(s) >= 6:
+            return s[3:-3]
+        elif len(s) >= 2:
+            return s[1:-1]
+        else:
+            return s
+    else:
+        # starts double
+        ends_double = s.endswith('"')
+        if not ends_double:
+            return s
+        elif s.startswith('"""') and s.endswith('"""') and len(s) >= 6:
+            return s[3:-3]
+        elif len(s) >= 2:
+            return s[1:-1]
+        else:
+            return s
+
+
 def check_for_partial_string(x):
     """Returns the starting index (inclusive), ending index (exclusive), and
     starting quote string of the most recent Python string found in the input.
@@ -2069,7 +2131,9 @@ def expandvars(path):
             name = match.group("envvar")
             if name in env:
                 ensurer = env.get_ensurer(name)
-                value = ensurer.detype(env[name])
+                val = env[name]
+                value = str(val) if ensurer.detype is None else ensurer.detype(val)
+                value = str(val) if value is None else value
                 path = POSIX_ENVVAR_REGEX.sub(value, path, count=1)
     return path
 
@@ -2294,6 +2358,9 @@ def columnize(elems, width=80, newline="\n"):
     return lines
 
 
+ALIAS_KWARG_NAMES = frozenset(["args", "stdin", "stdout", "stderr", "spec", "stack"])
+
+
 def unthreadable(f):
     """Decorator that specifies that a callable alias should be run only
     on the main thread process. This is often needed for debuggers and
